/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import java.util.Base64;

import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.user.OzgCloudKeycloakUserSpec.KeycloakUserSpecUser;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.api.model.SecretBuilder;

@Component
class UserSecretBuilder {

	static final String SECRET_TYPE = "Opaque";
	static final String SECRET_PASSWORD_FIELD = "password";
	static final String SECRET_NAME_FIELD = "name";

	public Secret build(String name, KeycloakUserSpecUser userSpec, String namespace) {
		return new SecretBuilder()
				.withType(SECRET_TYPE)
				.withMetadata(createMetaData(name, namespace))
				.addToStringData(SECRET_NAME_FIELD, userSpec.getUsername())
				.addToData(SECRET_PASSWORD_FIELD, generatePassword())
				.build();
	}

	String generatePassword() {
		var upperCaseCharacter = RandomStringUtils.randomAlphabetic(8).toUpperCase();
		var lowerCaseCharacter = RandomStringUtils.randomAlphabetic(8).toLowerCase();
		var numbers = RandomStringUtils.randomNumeric(8);
		return encode(upperCaseCharacter + lowerCaseCharacter + numbers);
	}

	String encode(String strValue) {
		return new String(Base64.getEncoder().encode(strValue.getBytes()));
	}

	private ObjectMeta createMetaData(String name, String namespace) {
		var metadata = new ObjectMeta();
		metadata.setName(name);
		metadata.setNamespace(namespace);

		return metadata;
	}
}