/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import java.util.List;

import de.ozgcloud.operator.keycloak.client.OzgCloudKeycloakClientSpec.ClientRole;

public class OzgCloudKeycloakClientSpecTestFactory {

	public static final String CLIENT_ID = "ClientName";
	public static final String BASE_URL = "https://BaseUrl.local";
	public static final String REDIRECT_URL = "https://test.local";
	public static final List<String> REDIRECT_URLS = List.of(REDIRECT_URL);
	public static final String WEB_ORIGIN = "https://weborigin.local";
	public static final List<String> WEB_ORIGINS = List.of(WEB_ORIGIN);

	public static final String ROLE_NAME_1 = "RoleName1";
	public static final ClientRole ROLE1 = ClientRole.builder().name(ROLE_NAME_1).build();
	public static final String ROLE_NAME_2 = "RoleName2";
	public static final ClientRole ROLE2 = ClientRole.builder().name(ROLE_NAME_2).build();
	public static final List<ClientRole> ROLES = List.of(ROLE1);

	public static final OzgCloudKeycloakClientSpec.ProtocolMapper PROTOCOL_MAPPER_1 = OzgCloudKeycloakClientSpecProtocolMapperTestFactory.create();

	public static final OzgCloudKeycloakClientSpec create() {
		return createBuilder().build();
	}

	public static final OzgCloudKeycloakClientSpec.OzgCloudKeycloakClientSpecBuilder createBuilder() {
		return OzgCloudKeycloakClientSpec.builder()
				.clientId(CLIENT_ID)
				.baseUrl(BASE_URL)
				.redirectUris(REDIRECT_URLS)
				.webOrigins(WEB_ORIGINS)
				.clientRoles(ROLES)
				.protocolMappers(List.of(PROTOCOL_MAPPER_1));
	}
}
